% Author: Lukasz A. Sterczewski, Ph.D.
% NASA Jet Propulsion Laboratory, California Institute of Technology
% email address: lukasz.a.sterczewski@jpl.nasa.gov 
% March 2020; Last revision: 24-March-2020

% This code is intended to demonstrate the algorithm described in 
% "Subsampling Dual-Comb Spectroscopy" by L. A. Sterczewski & M. Bagheri 

% It subsamples a real dual-comb signal, provides line numbering, and
% demonstrates the efficacy of coherent averaging of polyphase components

% This is a parent script. Use it to provide simulation parameters and run
% the plotting routine.

% Funding: National Aeronautics and Space Agencys (NASA) PICASSO program 
% (106822 / 811073.02.24.01.85);  Universities Space Research Association
% (USRA), NASA Postdoctoral Program (NPP). % 

% Acknowledgement: This work was supported under and was in part performed 
% at the Jet Propulsion Laboratory (JPL), California Institute of Technology,
% under contract with the NASA. L. A. Sterczewskis % research was supported 
% by an appointment to the NASA Postdoctoral Program % at JPL, administered 
% by Universities Space Research Association under contract with NASA. 

clear all;
close all;
clc;

addpath('.././data');
addpath('.././pathfcns');
addpath('.././pathfcns/cbrewer');
exportFontSize = 12;

saveFigures = false; %set to true if figure files are to be saved

load('ICL_DCS_harmonic.mat') % Real data

%% Signal parameters - user input

deltaF_r = 22.964 * 10^6; %Hz, comb repetition difference
f_max = 1610 * 10^6; %Hz

subsamplingVariant = 'plus'; %can be 'plus', 'minus' here, 'likedcs' is too demanding

M = 7; %decimation factor 7 is because the 10 dB linewidth is ~3.5 MHz and new lines 
       %must fit into the reduced spacing
nCombLines = floor(f_max ./ deltaF_r);

k = ceil(nCombLines ./ M);

switch subsamplingVariant
    case 'plus'
        idealFoldingFrequency = deltaF_r * (k+(1/(2*M))); 
        idealFs = M*2*idealFoldingFrequency;
    case 'minus'
        idealFoldingFrequency = deltaF_r * (k-(1/(2*M))); 
        idealFs = M*2*idealFoldingFrequency;
    otherwise
        error('Only ''plus'', ''minus'' variants are supported here');
end


%% Resampling to a new rate for decimation
[p,q] = rat(idealFs / Fs)
Yres = resample(Yres,p,q); %resampled DCS interferogram
Fs = round(Fs.*p/q); %new sample rata

%% Frequency domain analysis 
L = length(Yres);
nFFT = L;
frequencyAxis = linspace(0, 1, nFFT) * Fs;
Fs = round(Fs);
timeAxis = [0:L-1] * 1/Fs;

originalDcsSpectrum = abs(fft(Yres, nFFT) ./ L).^2;
selfMixingSpectrum = abs(fft((Yres.^2-mean(Yres.^2)), nFFT) ./ L).^2;

%Analyze with the pwelch periodogram averaging method
L1 = length(Yres);
nChunks = 100*M;
windowForPwelch = hann(round(L1 /nChunks));
nFFT = 2^nextpow2(10*L1/nChunks);
warning('off','MATLAB:colon:nonIntegerIndex')
[pxxOriginal, freqAxisOriginal] = pwelch(Yres, windowForPwelch, 0.7, nFFT, Fs, 'power');
plotOffset = -max(10*log10(pxxOriginal)); %dB

rbw = 1/(length(windowForPwelch)/Fs) * 1.5 %resolution bandwidth defined by the Hanning window

%% Plot the power spectrum
%Normal power spectrum 
myfigure(1000, 500);
plot(frequencyAxis(1:end/2)./10^6, originalDcsSpectrum(1:end/2) ./ max(originalDcsSpectrum(1:end/2)));
hold on;
plot(frequencyAxis(1:end/2)./10^6, selfMixingSpectrum(1:end/2) ./ max(selfMixingSpectrum(1:end/2)), 'r');
set(gca, 'YScale', 'log');
ylim([10^-4 1]); title('Power spectrum before subsampling');
xlabel('Frequency (MHz)'); ylabel('Normalized power (a.u.)');
legend('DCS spectrum', 'Self-mixing spectrum');

%PWelch spectrum (averaged periodograms)
myfigure(1200, 300); %myfigure(650, 170) in the paper
plot(freqAxisOriginal/10^6, plotOffset+10*log10(pxxOriginal), 'Color', 'k');

%Label the beat notes above
dF = mean(diff(freqAxisOriginal)); fSampleInterval = ceil(deltaF_r/dF);
bnIdx = [1:1:nCombLines] .* fSampleInterval;

if( bnIdx(end) > (nFFT/2) )
    warning('Some lines may be aliased! Try choosing the ''plus'' variant');
    bnIdx = bnIdx(1:end-1);
    nCombLines = nCombLines - 1;
end

bnAmps = 10*log10(pxxOriginal(bnIdx));

for iBn = 1:nCombLines
    if iBn < 10
        text(freqAxisOriginal(bnIdx(iBn))/10^6-10, bnAmps(iBn)+plotOffset+1, [' ', num2str(iBn)], 'Fontsize', 7)
    else
        text(freqAxisOriginal(bnIdx(iBn))/10^6-10, bnAmps(iBn)+plotOffset+1, num2str(iBn), 'Fontsize', 7)
    end
end

xlim([0 1620])
ylim([-40 3])
xlabel('Frequency (MHz'); ylabel('Relative power (dB)');
a = gca;
a.YTick = [-30:10:0];
tightfig;

if(saveFigures)
    print('normalSpectrum.pdf', '-dpdf')
end

%% Subsampling and coherent averaging
%Polyphase decomposition
for iPhaseOffset = 1:M
    try
        signalDecimated(iPhaseOffset, :) = downsample(Yres, M, iPhaseOffset-1);
    catch
        signalDecimated(iPhaseOffset, :) = [downsample(Yres, M, iPhaseOffset-1) 0];
    end
    
end
signalDecimated = signalDecimated(:, 1:end-1);

%Estimation of the polyphase components delay. It can be obtained analytically
delay_estim = finddelay(signalDecimated(1,:), signalDecimated(2,:))
delay_for_each_polyphase_comp = mod([0:M-1].*delay_estim, 141);

%Delay each component
signal_downsampled_shifted = [];
for iPhaseOffset = 1:M
    signal_downsampled_shifted(iPhaseOffset, :) = circshift(signalDecimated(iPhaseOffset,:), -delay_for_each_polyphase_comp(iPhaseOffset));
end

%Calculate an averaged time-domain trace for noise removal
%For M = 7, we are expecting a sqrt(7)x SNR improvement, which is
%log2(sqrt(7) extra bits
averaged_signal = mean(signal_downsampled_shifted,1);

%% Plot the original time-domain signal along with its subsampled representation
myfigure(1200, 600);
ax1 = subplot(2,1,1); hold on;
plot(timeAxis*10^6, Yres, 'linewidth', 1, 'Color', [.8 .8 .8]);
plot(timeAxis(1:M:end-M)*10^6, [signalDecimated(1,:)], 'linewidth', 1, 'Marker', '.', 'MarkerSize', 12);
plot(timeAxis(1:M:end-M)*10^6, [signalDecimated(2,:)], 'linewidth', 1, 'Marker', '.', 'MarkerSize', 12);
xlim([0 0.4]); box on;
xlabel('Time (\mus)'); ylabel('Voltage (a.u.)');
legend('x', 'x_{p1}', 'x_{p2}', 'location', 'se');
title('Polyphase decomposition through sample-delay and decimation (phase 1 & 2)');

ax2 = subplot(2,1,2); hold on;
plot(timeAxis(1:M:end-M)*10^6, signal_downsampled_shifted', 'Color', [.8 .8 .8]);
plot(timeAxis(1:M:end-M)*10^6, averaged_signal, 'Linewidth', 1.5, 'Color', 'k'); 
title('Aligned polyphase components and their average');
xlim([0 0.4]);  box on;
xlabel('Time (\mus)'); ylabel('Voltage (a.u.)');
linkaxes([ax1,ax2],'x');
tightfig;

%% Frequency domain analysis of the subsampled signal
%Normal power spectrum
hannWinForDecimatedSignal = hann(length(signalDecimated));
fftSubsampled = abs(fft(hannWinForDecimatedSignal'.*signalDecimated(1,:), nFFT) ./ length(signalDecimated)).^2;
fftSubsampledAveraged = abs(fft(hannWinForDecimatedSignal'.*averaged_signal, nFFT) ./ length(signalDecimated)).^2;
fAxisSubsampled = linspace(0,Fs/M, nFFT);

%Perform PWelch analysis:
%Analyze with the pwelch periodogram averaging method
nChunks = 100*M;
windowForPwelch = hann(round(L1 /nChunks));
windowForPwelchLong = hann(round(L1 /(nChunks/M)));
nFFT = 2^nextpow2(10*L1/nChunks);

%coherently averaged first:
[pxxCoh, ~] = pwelch(averaged_signal, windowForPwelch, 0.7, nFFT, Fs/M, 'power');
plotOffsetSubsampled = -max(10*log10(pxxCoh)); %dB; %dB
rbw = 1/(length(windowForPwelch)/Fs)*1.5;

%incoherently averaged:
[pxxIncoh, ~] = pwelch(signalDecimated', windowForPwelch, 0.7, nFFT, Fs/M, 'power');
pxxIncoh = mean(pxxIncoh');

%Original signal:
[pxxSingle, freqAxisSub] = pwelch(signalDecimated(1,:), windowForPwelch, 0.7, nFFT, Fs/M, 'power');

%% Plot the subsampled power spectrum
%Normal power spectrum
myfigure(1000, 500);
plot(fAxisSubsampled(1:end/2)./10^6, (fftSubsampled(1:end/2) ./ max(fftSubsampled(1:end/2))));
hold on;
plot(fAxisSubsampled(1:end/2)./10^6, (fftSubsampledAveraged(1:end/2) ./ max(fftSubsampledAveraged(1:end/2))), 'r');
set(gca, 'YScale', 'log');
ylim([10^-4 3]); title('Power spectrum after subsampling');
xlabel('Frequency (MHz)'); ylabel('Normalized power (a.u.)');
legend('Spectrum of a single-tap output', 'Spectrum of the average');
xlim([0 fAxisSubsampled(end/2)./10^6]);

%PWelch periodogram
myfigure(1200, 300); %myfigure(650, 170) in the paper
plot(freqAxisSub/10^6, plotOffsetSubsampled + 10*log10(pxxCoh), 'Color', 'k');
hold on;
plot(freqAxisSub/10^6, plotOffsetSubsampled + 10*log10(pxxSingle), 'Color', [0 0.45 0.74]);
plot(freqAxisSub/10^6,  plotOffsetSubsampled + 10*log10(pxxIncoh), 'Color', [0.85 0.33 0.1]);
legend('Coherently averaged polyphase outputs', 'Single-tap output', 'Power-averaged polyphase outputs', 'location', 's');

%Label beat notes
aliasOrder = -M:1:M;
fn = [];
for iLine = 1:nCombLines
    allFrequencies = abs( iLine .* deltaF_r + aliasOrder .* (Fs/M) );
    freq = unique(allFrequencies( allFrequencies <= Fs/(2*M)));
    f_n(iLine,:) = freq;
end

for iBn = 1:nCombLines
    if iBn < 10
        text(f_n(iBn)/10^6-1, 0, [' ',num2str(iBn)], 'Fontsize', 7);
    else
        text(f_n(iBn)/10^6-1, 0, num2str(iBn), 'Fontsize', 7);
    end
end

ylim([-47 5])
xlim([0 freqAxisSub(round(nFFT/2))/10^6])
xlabel('Frequency (MHz'); ylabel('Relative power (dB)');
a = gca;
a.YTick = [-40:10:0];
tightfig;

if(saveFigures)
    print('compressedAveragedSpectrum.pdf', '-dpdf')
end
