% Author: Lukasz A. Sterczewski, Ph.D.
% NASA Jet Propulsion Laboratory, California Institute of Technology
% email address: lukasz.a.sterczewski@jpl.nasa.gov 
% March 2020; Last revision: 24-March-2020

% This code is executed using combSimulator_XXX.m files. Do not run it by
% itself!

% Funding: National Aeronautics and Space Agencys (NASA) PICASSO program 
% (106822 / 811073.02.24.01.85);  Universities Space Research Association
% (USRA), NASA Postdoctoral Program (NPP). 

% Acknowledgement: This work was supported under and was in part performed 
% at the Jet Propulsion Laboratory (JPL), California Institute of Technology,
% under contract with the NASA. L. A. Sterczewskis % research was supported 
% by an appointment to the NASA Postdoctoral Program % at JPL, administered 
% by Universities Space Research Association under contract with NASA. 

%% Calculation of the sampling frequency - here sub-sampling is realized via decimation

%The total electrical bandwidth is nCombLines*deltaF_r, we must pick a
%sampling frequency greater than that, and such that is M times larger than
%the line spacing
f_max = nCombLines * deltaF_r; %MHz
k = ceil(nCombLines ./ M); %the order of the rf comb line around which folding will occur

switch subsamplingVariant
    case 'plus'
        idealFoldingFrequency = deltaF_r * (k+(1/(2*M))); 
        Fs_demo = M*2*idealFoldingFrequency;
    case 'minus'
        idealFoldingFrequency = deltaF_r * (k-(1/(2*M))); 
        Fs_demo = M*2*idealFoldingFrequency;
    case 'likedcs'
        if M < (2*(nCombLines + 1))
            error('In DCS-like compression, the decimation factor must be at least twice (the number of DCS lines + 1)');
        end
        idealFoldingFrequency = (deltaF_r * M-1); 
        Fs_demo = deltaF_r * (M-1);
    otherwise
        error('Only ''plus'', ''minus'', or ''likedcs'' variants are supported');
end

nPossibleCombLines = floor(Fs_demo/(2*deltaF_r));

if( f_max > Fs_demo/2)
    error(['Too many lines in the dual-comb spectrum to subsample. Consider lowering it, choose ' ...
        'a lower decimation factor M or pick the higher (plus) sampling frequency variant F_S+. ' ...
        'Currently one can fit ' num2str(nPossibleCombLines), ' lines, while you want ', num2str(nCombLines), '.']);
end

%Determine the folding Axis and new spectrum parameters
foldingAxis = Fs_demo / (2*M);
expectedLineSpacing = deltaF_r / M;
deviationFromEquidistance = foldingAxis - idealFoldingFrequency;

fprintf('==========================================================\n')
fprintf('Subsampling in dual-comb spectroscopy - demonstration code\n')
fprintf('----------------------------------------------------------\n')
fprintf('The unaliased comb has %i lines spaced by: %.3f MHz\n', nCombLines, deltaF_r/10^6);
fprintf('__________________________________________________________\n')
fprintf('The expected comb tooth spacing is: %.3f MHz\n', expectedLineSpacing/10^6);
fprintf('The expected folding axis should be located at: %.3f MHz\n', idealFoldingFrequency/10^6);
fprintf('The spectral folding axis will be located at: %.3f MHz\n', foldingAxis/10^6);
fprintf('The deviation from line equidistance is therefore: %.3f kHz\n', deviationFromEquidistance/10^3);

%% Generate signal
timeAxis = [0:nIfgSamples] * 1/Fs_demo;
dcsSignal = zeros(size(timeAxis));

rng(4) %seed the random number generator

%Random phase comb model (FM-like comb)
for iLine=1:nCombLines
    initPhase = rand() * 2* pi; 
    lineIntensity = 1+sqrt(iLine);
    dcsSignal = dcsSignal +  real(lineIntensity .* exp(1i.*(initPhase)) .* exp(1i .*(2.*pi.*iLine.*deltaF_r.*timeAxis)));
end

myfigure(800, 600);
ax1 = subplot(2,1,1);
plot(timeAxis, dcsSignal);
xlim([0 10^-1*timeAxis(end)]);
xlabel('Time (s)');
title('Unaliased dual-comb beating signal');

ax2 = subplot(2,1,2);
plot(timeAxis(1:M:end), dcsSignal(1:M:end), 'Color', 'r');
xlim([0 10^-1*timeAxis(end)]);
title('Subsampled dual-comb beating signal');
xlabel('Time (s)')
linkaxes([ax1,ax2],'x');


%% Frequency domain analysis
L = length(dcsSignal);
nFFT = L*10;
frequencyAxis = linspace(0, 1, nFFT) * Fs_demo;
Fs_demo = round(Fs_demo);

dcsSpectrum = abs(fft(dcsSignal, nFFT)).^2;
nVector = 0:nCombLines;
offsetFreeFAxis = (0:1:nCombLines) * deltaF_r;

%% Plot basic version:
myfigure(1200, 400);
plot(frequencyAxis, dcsSpectrum ./ max(dcsSpectrum));
xticks(offsetFreeFAxis);
xticklabels(nVector);
xlabel('Beat note number');
ylabel('Power (a.u.)');
title('Original dual-comb spectrum in [0, Fs]');


%% Paper figure - plot 1 (original mapping)
myfigure(1200, 300);
xlabel('Beat note number'); 
ylabel('Power (a.u)');

hold on;
%Plot in a loop - segmented color
nSamplesAround = 100;

cm = cbrewer('div', 'Spectral',length(nVector)-1);
cm = rgb2hsv(cm);
cm(:,3) = 0.7;
cm = hsv2rgb(cm);
for iBeatNote = 1:length(nVector)-1
    %Find the index of the peak
    frequencyLocIdx = round( (iBeatNote * deltaF_r) / (Fs_demo/nFFT));
    idxRange = frequencyLocIdx-nSamplesAround/2 : frequencyLocIdx+nSamplesAround/2;
    plot( frequencyAxis( idxRange ), dcsSpectrum(idxRange) ./ max((dcsSpectrum)), 'Linewidth', 2, 'Color', cm(iBeatNote,:));
end
    
box on;    
a = gca();
set(a, 'Fontname', 'Tahoma', 'Fontsize', exportFontSize);
topTickPositions = a.XTick;
topTickLabels = (topTickPositions /10^6);
a2 = LinkTopAxisData(topTickPositions,topTickLabels,'Frequency (MHz)'); % Add a top axis
a2.FontSize = exportFontSize;
a2.FontName = 'Tahoma';
axis(a); 
a.XTick = offsetFreeFAxis;
a.XTickLabels = nVector;
a.XLim = [0 1.02*max(offsetFreeFAxis)];
tightfig;
if(saveFigures)
    print('MappingOrig.emf', '-dmeta');
    print('-painters', '-dpdf', 'MappingOrig.pdf');
end

%% Plot in a cyclic representation (on a unit circle)
myfigure(800, 700);
theta = linspace(0, 2*pi, length(frequencyAxis));
X = cos(theta);
Y = sin(theta);

for iBeatNote = 1:length(nVector)-1
    %Find the index of the peak
    frequencyLocIdx = round( (iBeatNote * deltaF_r) / (Fs_demo/nFFT));
    idxRange = frequencyLocIdx-nSamplesAround/2 : frequencyLocIdx+nSamplesAround/2;
    plot3(X(idxRange),Y(idxRange),dcsSpectrum(idxRange) ./ max(dcsSpectrum ), 'Linewidth', 2, 'Color', cm(iBeatNote,:));
    hold on;
    plot3(X(end-idxRange),Y(end-idxRange),dcsSpectrum(end-idxRange) ./ max(dcsSpectrum ), 'Linewidth', 2, 'Color', cm(iBeatNote,:));
    text(1.05*X(frequencyLocIdx), 1.1*Y(frequencyLocIdx), num2str(iBeatNote))
 end

plot3(X,Y,zeros(size(X)), 'Color', 'k');
set(gca, 'Projection','perspective');
grid off;
set(gca,'visible','off')
view([-164 50])
tightfig;
if(saveFigures)
    print('Circular.emf', '-dmeta');
    print('Circular.pdf', '-dpdf');
    print('Circular.svg', '-dsvg');
end

%% Subsampling (sample decimation)
subsampledDcsSignal = downsample(dcsSignal,M);
fAxisDown = linspace(0, 1, nFFT/M) * Fs_demo/M;
subsampledDcsSpectrum = abs(fft(subsampledDcsSignal, round(nFFT/M) )).^2; %power spectrum
nDetectedPeaks = length(findpeaks(subsampledDcsSpectrum, 'MinPeakHeight', 0.05*max(subsampledDcsSpectrum)))/2;

%Generate the sub-sampled frequency axis, extract values lower than Fs/decimFactor
aliasOrder = -M:1:M;
fn = [];
for iLine = 0:nCombLines
    allFrequencies = abs( iLine .* deltaF_r + aliasOrder .* (Fs_demo/M) );
    freq = unique(allFrequencies( allFrequencies <= Fs_demo/(2*M)));
    f_n(iLine+1,:) = freq;
end

%Plot frequency mapping
myfigure(800, 400);
scatter(nVector, f_n/10^6, 'filled');
xlabel('Beat note number');
ylabel('New frequency (MHz)');
title('Mapping of subsampled comb lines');
box on;

%% Plot folded spectrum
myfigure(1200, 300);
xlabel('Beat note number'); 
ylabel('Power (a.u)');
hold on;
%Plot in a loop - segmented color
nSamplesAround = 100;

cm = cbrewer('div', 'Spectral',length(nVector));
cm = rgb2hsv(cm);
cm(:,3) = 0.7;
cm = hsv2rgb(cm);
for iBeatNote = 2:length(nVector)
    %Find the index of the peak
    frequencyLocIdx = round(f_n(iBeatNote) / (Fs_demo/nFFT));
    idxRange = frequencyLocIdx-nSamplesAround/2 : frequencyLocIdx+nSamplesAround/2;
    plot( fAxisDown( idxRange ), subsampledDcsSpectrum(idxRange) ./ max((subsampledDcsSpectrum)), 'Linewidth', 1.5, 'Color', cm(iBeatNote,:));
end
box on;    
a = gca;
set(gca(), 'Fontname', 'Tahoma', 'Fontsize', exportFontSize);
topTickPositions = a.XTick;
topTickLabels = (topTickPositions /10^6);
a2 = LinkTopAxisData(topTickPositions,topTickLabels,'Frequency (MHz)'); % Add a top axis
a2.FontSize = exportFontSize;
a2.FontName = 'Tahoma';
axis(a); 
[sortedFAxis,idx] = sort(f_n);
a.XTick = sortedFAxis;
a.XTickLabels = nVector(idx);
xlim([0 1.02*max(f_n)])
tightfig;

if(saveFigures)
    print('Mapping.emf', '-dmeta');
    print('Mapping.pdf', '-dpdf');
end

%% Plot the folded spectrum in 3D (wrapped)
myfigure(800, 700);
theta = linspace(0, M * 2*pi, length(frequencyAxis));
radius = 1 - 0.007 * (linspace(0, M, length(frequencyAxis)));
X = radius .* cos(theta);
Y = radius .* sin(theta);

for iBeatNote = 1:length(nVector)-1
    %Find the index of the peak
    frequencyLocIdx = round((iBeatNote * deltaF_r) / (Fs_demo/nFFT));
    idxRange = frequencyLocIdx - nSamplesAround/2 : frequencyLocIdx + nSamplesAround/2;
    plot3( X(idxRange) ,Y(idxRange), dcsSpectrum(idxRange) ./ max(dcsSpectrum), 'Linewidth', 2, 'Color', cm(iBeatNote,:));
    hold on;
    plot3( X(end-idxRange), Y(end-idxRange), dcsSpectrum(end-idxRange) ./ max(dcsSpectrum), 'Linewidth', 2, 'Color', cm(iBeatNote,:));
    text(1.05*X(frequencyLocIdx), 1.1 * Y(frequencyLocIdx), num2str(iBeatNote));
end

plot3( X,Y,zeros(size(X)), 'Color', [0.8 0.8 0.8], 'linewidth', 0.5 );
set(gca, 'Projection','perspective');
grid off;
set(gca,'visible','off')
view([-164 50])
tightfig;

if(saveFigures)
    print('-painters', '-dpdf', 'Circular_wrap.pdf');
end